use crate::Numeric;
pub use self::generic::{ColumnVector, GenericMatrix, RowVector, SquareMatrix};

mod generic;

/// Abstraction for all kinds of fixed size matrices.
pub trait Matrix<T: Numeric, const ROWS: usize, const COLUMNS: usize> {
    /// Get the value of the given cell.
    ///
    /// # Panics
    ///
    /// This function panics, if the row or column are outside of the valid range of the matrix.
    fn get(&self, row: usize, col: usize) -> T;

    /// Set the value of the given cell.
    ///
    /// # Panics
    ///
    /// This function panics, if the row or column are outsize of the valid range of the matrix.
    ///
    /// Additionally it may panic, if the value is invalid for the given cell.
    /// E.g. cells outsize of the main diagonal of a `DiagonalMatrix` can only be set to zero.
    fn set(&mut self, row: usize, col: usize, value: T);
}
